# to run this script you need a DL1 debug files of hipeRTA and a DL1 file from lstchain from the same run


import numpy as np
import matplotlib.pyplot as plt
from ctapipe.visualization import CameraDisplay
from ctapipe.image import tailcuts_clean
from lstchain.io.io import (
    dl1_images_lstcam_key,
    dl1_params_lstcam_key,
    read_camera_geometries,
)
from astropy.table import Table, join
from ctapipe.containers import HillasParametersContainer
from matplotlib.backends.backend_pdf import PdfPages
import astropy.units as u
from astropy.coordinates import Angle
from lstchain.io.config import get_standard_config
from lstchain.io.config import read_configuration_file


def get_hillas_container(row):
    h = HillasParametersContainer()
    h.x = row["x"] * 28 * u.m
    h.y = row["y"] * 28 * u.m
    h.r = row["r"] * 28 * u.m
    h.phi = Angle(row["phi"] * u.rad)
    h.width = row["width"] * u.m
    h.length = row["length"] * u.m
    h.psi = Angle(row["psi"] * u.rad)
    h.skewness = row["skewness"]
    h.kurtosis = row["kurtosis"]
    h.length_uncertainty = 0 * u.m
    h.width_uncertainty = 0 * u.m
    return h


def get_cleaning_config(config_file=None):
    if config_file is None:
        config = get_standard_config()
    else:
        config = read_configuration_file(config_file)
    cleaning_parameters = config["tailcut"]
    cleaning_parameters.pop("use_only_main_island", True)
    cleaning_parameters.pop("delta_time", None)
    return cleaning_parameters


def main(filename, config_file=None):

    geom = read_camera_geometries(filename)["LSTCam"]

    dl1_parameters_table = Table.read(filename, path=dl1_params_lstcam_key)
    images_table = Table.read(filename, path=dl1_images_lstcam_key)

    dl1_table = join(
        dl1_parameters_table, images_table, keys=["event_id", "tel_id", "obs_id"]
    )

    params_cleaning = get_cleaning_config(config_file)

    selected_table = dl1_table[np.isfinite(dl1_table["intensity"])]
    selected_table = dl1_table[dl1_table["intensity"] > 500]

    with PdfPages("images_examples.pdf") as pp:

        for ii, row in enumerate(selected_table[:10]):
            h = get_hillas_container(row)

            image = row["image"]
            peak_time = row["peak_time"]

            clean_mask = tailcuts_clean(geom, image, **params_cleaning)

            fig, axes = plt.subplots(1, 2, figsize=(12, 6))
            fig.suptitle(f"event id : {row['event_id']}")

            ax = axes[0]
            display = CameraDisplay(geom, image, ax=ax)
            display.add_colorbar(ax=ax)
            ax.set_title("charges")
            display.highlight_pixels(clean_mask, color="red", alpha=0.33)
            display.overlay_moments(h)

            ax = axes[1]
            display = CameraDisplay(geom, peak_time, ax=ax)
            display.highlight_pixels(clean_mask, color="red", alpha=0.2)
            display.add_colorbar(ax=ax)
            ax.set_title("peak time")

            pp.savefig(dpi=100)


if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(description="Images debug")
    parser.add_argument("filename", type=str, help="path to the lstchain DL1 file")

    args = parser.parse_args()

    main(args.filename)



#!/usr/bin/env python

from ruamel.yaml import YAML
import calendar
import logging

log = logging.getLogger(__name__)

    

def load_config(config_path):
    """
    Load the pipeline config, test for invalid values and
    set paths to the data files/directories.

    Parameters:
    -----------
    config_path: str or Path-like object
        Path to the config file

    Returns:
    --------
    config: dict
        Dictionary with parameters for the r0_to_dl3 processing
    """

    # This could easily be adapted to support different formats
    with open(config_path) as f:
        loaded_config = YAML().load(f)

    config_valid(loaded_config)

    config = complete_lstmcpipe_config(loaded_config)

    log.info(
        f'************ - lstMCpipe will be launch using the {config["workflow_kind"]} pipeline:- ************'
    )
    log.info(f'\nPROD_ID to be used: {config["prod_id"]}')
    log.info("\nStages to be run:\n - " + "\n - ".join(config["stages_to_run"]))


    log.info(
        "Slurm configuration:"
        + f"\n - Source environment: {config['batch_config']['source_environment']}"
        + f"\n - Slurm account: {config['batch_config']['slurm_account']}"
    )

    log.warning(
        "\n! Subdirectories with the same PROD_ID and analysed the same day will be overwritten !"
    )

    return config


def config_valid(loaded_config):
    """
    Test if the given dictionary contains valid values for the
    r0_to_dl3 processing.
    TODO: The stages should be checked aswell.
    Not all combinations are sensible!

    Parameters:
    -----------
    loaded_config: dict
        Dictionary with the values in the config file

    Returns:
    --------
    True if config is valid
    """
    # Allowed options
    compulsory_entries = [
        "workflow_kind",
        "source_environment",
        "stages_to_run",
        "stages",
        # TODO dl1_reference_id ?
    ]
    allowed_workflows = ["hiperta", "lstchain", "ctapipe"]


    # Check allowed cases
    for item in compulsory_entries:
        if item not in loaded_config:
            raise Exception(
                f"The lstMCpipe configuration was not generated correctly. \n"
                f"Missing: {item} key"
            )

    workflow_kind = loaded_config["workflow_kind"]

    if workflow_kind not in allowed_workflows:
        raise Exception(
            f"Please select an allowed `workflow_kind`: {allowed_workflows}"
        )


    stages_to_be_run = loaded_config["stages_to_run"]
    if "dl1ab" in stages_to_be_run:
        if not "dl1_reference_id" in loaded_config:
            raise KeyError(
                "The key dl1_reference_id has to be set in order to locate "
                "the input files for the dl1ab stage"
            )

    dl1_noise_tune_data_run = loaded_config.get("dl1_noise_tune_data_run")
    dl1_noise_tune_mc_run = loaded_config.get("dl1_noise_tune_mc_run")
    if dl1_noise_tune_data_run and not dl1_noise_tune_mc_run:
        raise KeyError(
            "Please specify a simtel monte carlo file to "
            "compare observed noise against."
        )
    elif not dl1_noise_tune_data_run and dl1_noise_tune_mc_run:
        raise KeyError("Please specify an observed dl1 file to " "tune the images.")

    log.debug("Configuration deemed valid")

    return True


def complete_lstmcpipe_config(loaded_config):
    """
    Completes (and get default values) of some entries of the lstmcpipe config

    Parameters
    ----------
    loaded_config: dict
        Loaded config generated by lstmcpipe PathConfig

    Returns
    -------
    config : dict
        Dictionary with all the key:values completed along the r0_to_dl3 workflow

    """
    config = loaded_config.copy()

    suffix_prod_id = loaded_config.get("prod_id", "v00")
    workflow_kind = loaded_config["workflow_kind"]

    # TODO ??
    # # to locate the source dl1 files
    # dl1_reference_id = loaded_config.get("dl1_reference_id")
    # # Full path to an observed dl1 file
    # dl1_noise_tune_data_run = loaded_config.get("dl1_noise_tune_data_run")
    # dl1_noise_tune_mc_run = loaded_config.get("dl1_noise_tune_mc_run")

    # Prod_id syntax
    t = calendar.datetime.date.today()
    year, month, day = f"{t.year:04d}", f"{t.month:02d}", f"{t.day:02d}"
    if workflow_kind == "lstchain":

        import lstchain
        base_prod_id = f"{year}{month}{day}_v{lstchain.__version__}"

    elif workflow_kind == "ctapipe":

        import ctapipe
        base_prod_id = f"{year}{month}{day}_vctapipe{ctapipe.__version__}"

    elif workflow_kind == "hiperta":  # RTA

        # TODO parse version from hiPeRTA module
        import lstchain
        base_prod_id = f"{year}{month}{day}_vRTA420_v{lstchain.__version__}"

    # Create the final config structure to be passed to the pipeline
    # 1 - Prod_id
    prod_type = f"_{loaded_config['prod_type']}" if 'prod_type' in loaded_config else ''
    suffix_id = "{}_{}".format(prod_type, suffix_prod_id)
    config["prod_id"] = base_prod_id + suffix_id

    # 2 - Parse source environment correctly
    src_env = (
        f"source {loaded_config['source_environment']['source_file']}; "
        f"conda activate {loaded_config['source_environment']['conda_env']}; "
    )
    # 2.1 - Parse slurm user config account
    slurm_account = loaded_config.get("slurm_config", {}).get("user_account", "")

    # 2.2 - Create a dict for all env configuration and slurm configuration (batch arguments)
    config["batch_config"] = {
        "source_environment": src_env,
        "slurm_account": slurm_account,
    }

    # TODO ??
    # config["dl1_reference_id"] = dl1_reference_id
    # config["dl1_noise_tune_data_run"] = dl1_noise_tune_data_run
    # config["dl1_noise_tune_mc_run"] = dl1_noise_tune_mc_run

    return config



import numpy as np
import astropy.units as u
import matplotlib.pyplot as plt

from pyirf.utils import calculate_theta, calculate_source_fov_offset
from lstchain.io.io import read_mc_dl2_to_QTable
from lstchain.reco.utils import filter_events
from astropy.table import QTable


from ..scripts.script_dl2_to_sensitivity import determine_source_position, filters



def read_gh_cut_table(filename):
    return QTable.read(filename, hdu="GH_CUTS")

def read_theta_cut_table(filename):
    return QTable.read(filename, hdu="RAD_MAX")[0]



def theta2_hist_per_energy_bin(irf_file, dl2_gamma_file):
    """
    plot a theta2 histogram per energy bin of gammas selected event (passing gh_score cut)
    and displaying the theta2 cut applied for IRFs
    """

    gammas, sim_info = read_mc_dl2_to_QTable(dl2_gamma_file)
    gammas = filter_events(gammas, filters)
    for prefix in ("true", "reco"):
            k = f"{prefix}_source_fov_offset"
            gammas[k] = calculate_source_fov_offset(gammas, prefix=prefix)

    source_alt, source_az = determine_source_position(gammas)
    gammas['theta'] = calculate_theta(gammas, assumed_source_az=source_az, assumed_source_alt=source_alt)

    gh_cuts = read_gh_cut_table(irf_file)
    theta_cuts = read_theta_cut_table(irf_file)

    tc = theta_cuts["RAD_MAX"].T[:, 0]
    energy_min = theta_cuts['ENERG_LO']
    energy_max = theta_cuts['ENERG_HI']

    np.testing.assert_allclose(energy_min, gh_cuts['low'])
    np.testing.assert_allclose(energy_max, gh_cuts['high'])
    
    ncols = 5
    nrows = len(energy_min) // ncols + int((len(energy_min) % ncols)>0)
    
    fig, axes = plt.subplots(ncols=ncols, nrows=nrows, figsize=(ncols*5, nrows*5))
    xrange = (0,0.4)
    for ii, emin in enumerate(energy_min):
        ax=axes.ravel()[ii]
        emax = energy_max[ii]
        mask = (gammas['gh_score'] < gh_cuts['cut'][ii]) & (emin <= gammas['true_energy']) & (gammas['true_energy'] < emax)

        t2unit=u.deg**2
        n, bins, _ = ax.hist((gammas[mask]['theta']**2).to_value(t2unit),
                             label=f'{emin:0.2f}-{emax:0.2f}',
                             histtype='step',
                             lw=2,
                             bins=np.linspace(*xrange),
                             range=xrange,
                             density=False,
                )
        ax.vlines((tc[ii]**2).to_value(t2unit), 0, np.max(n), color='orange')
        ax.legend()
        ax.set_xlim(*xrange)
        ax.set_xlabel(f'theta2 / {t2unit}')
        ax.set_title(f"gh cut: {gh_cuts['cut'][ii]:0.3f}")
        plt.tight_layout()

    return axes



#!/usr/bin/env python3
#
# usage:
# python hiperta_r0_to_dl1lstchain.py -i INFILE [-o OUTDIR] [-c CONFIG_FILE] [-k KEEP_FILE]

import os
import argparse
from lstmcpipe.hiperta.reorganize_dl1hiperta300_to_dl1lstchain060 import (
    main as reorganize_dl1,
)


def main():
    """Run hiperta_r0_dl1 and reorganize_dl1hipertaV300_to_dl1lstchain060"""
    parser = argparse.ArgumentParser(
        description="Run hiperta_r0_dl1 and reorganize_dl1hiperta_v300_to_dl1lstchain_v060"
    )

    parser.add_argument(
        "--infile",
        "-i",
        type=str,
        dest="infile",
        help="mc r0 file to be run with hiperta_r0_dl1",
        required=True
    )

    parser.add_argument(
        "--outdir",
        "-o",
        type=str,
        dest="outdir",
        help="Path where to store the dl1 files.",
        default="./dl1_data/",
    )

    parser.add_argument(
        "--config",
        "-c",
        type=str,
        dest="config",
        help="Configuration file for hiperta_r1_dl1",
        default="./default_PConfigCut.txt",
        # TODO : set a way of creating the small config file that it is needed
    )

    parser.add_argument(
        "--keep_file",
        "-k",
        action='store_true',
        dest="keep_file",
        help="Keep output of hiperta. Set by default to False",
    )

    parser.add_argument(
        "--debug_mode",
        "-d",
        action='store_true',
        dest="debug_mode",
        help="Activate debug mode (add cleaned mask in the output hdf5). Set by default to False",
    )
    args = parser.parse_args()

    os.makedirs(args.outdir, exist_ok=True)

    # TODO. Hardcoded. Change `--selectedtel 1` when various tels available
    cmd_hiperta = f"hiperta_r0_dl1 -i {args.infile} -c {args.config} -o {args.outdir} --selectedtel 1"
    if args.debug_mode:  # in HiPeRTA
        cmd_hiperta += " -g"
    os.system(cmd_hiperta)

    # We know in advance the name of the output
    output_hiperta_filename = os.path.join(args.outdir, "dl1_" + os.path.basename(args.infile))
    output_reorganized_filename = os.path.join(
        args.outdir, "dl1v06_reorganized_" + os.path.basename(args.infile)
    )
    reorganize_dl1(output_hiperta_filename, output_reorganized_filename)

    # Erase the hiperta dl1 file created ?
    if not args.keep_file:
        os.remove(output_hiperta_filename)

    print("\nDone.")


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import shutil
import argparse
from pathlib import Path
from sklearn.model_selection import train_test_split

parser = argparse.ArgumentParser(
    description="Script to move a directory and its content after creating the destination"
    " directory."
)

parser.add_argument(
    "--input_dir",
    "-i",
    type=Path,
    dest="input_dir",
    help="Splitting input dir",
)

parser.add_argument(
    "--outdir_test",
    "--otest",
    type=Path,
    dest="outdir_test",
    help="Output directory for test dataset",
)

parser.add_argument(
    "--outdir_train",
    "--otrain",
    type=Path,
    dest="outdir_train",
    help="Output directory for train dataset",
)

parser.add_argument(
    "--ratio",
    "-r",
    type=float,
    dest="ratio",
    help="Train-test splitting ratio",
    default=0.5
)

parser.add_argument(
    "--log_dir",
    "-l",
    type=Path,
    dest="log_dir",
    help="Directory to store training and testing filelists"
)


def write_filelist(filelist, outdir, dataset=""):
    """
    Write all files within filelist in `{dataset}`.list file at outdir

    Parameters
    ----------
    filelist : list
        list of files to be written
    outdir : Path
        Output directory Path
    dataset : str
        'training' or 'testing' dataset list
    """
    with open(outdir.joinpath(f"{dataset}.list"), "w+") as newfile:
        for file in filelist:
            newfile.write(file)
            newfile.write("\n")


def move_files(filelist, outdir):
    """
    Move all files within filelist to outdir

    Parameters
    ----------
    filelist : list
        list of files to be written
    outdir : Path
        Output directory Path
    """
    for file in filelist:
        shutil.move(file, outdir)


def main():
    args = parser.parse_args()

    file_list = [file.resolve().as_posix() for file in Path(args.input_dir).iterdir()
                 if file.is_file() and file.name.endswith(".h5")]

    train, test = train_test_split(file_list,
                                   random_state=42,
                                   test_size=float(args.ratio),
                                   shuffle=True
                                   )

    write_filelist(train, args.log_dir, dataset="training")
    write_filelist(test, args.log_dir, dataset="testing")
    move_files(train, args.outdir_train)
    move_files(test, args.outdir_test)


if __name__ == "__main__":
    main()



# !/usr/bin/env python3

# Everything is hardcoded to LST telescopes - because this is just a temporal work that will no longer be used
# in the moment lstchain upgrades to V0.8

import os
import tables
import argparse
import numpy as np
from astropy.table import Table, vstack, join
from astropy.io.misc.hdf5 import write_table_hdf5
from lstmcpipe.hiperta.reorganize_dl1hiperta_to_dl1lstchain import (
    add_disp_and_mc_type_to_parameters_table,
)

parser = argparse.ArgumentParser(
    description="Re-organize the dl1 `standard` output file from either the "
    "hiptecta_r1_to_dl1 or hiperta_r1_dl1 to the lstchain DL1 structure"
)

parser.add_argument(
    "--infile",
    "-i",
    type=str,
    dest="infile",
    help="dl1 output file of `hiperta_r0_dl1` to be converted to dl1lstchain_v060",
    default=None,
)

parser.add_argument(
    "--outfile",
    "-o",
    type=str,
    dest="outfile",
    help="Output filename. dl1_reorganized.h5 by default.",
    default="./dl1v0.6_reorganized.h5",
)


def stack_and_write_images_table(input_filename, hfile_out, node_dl1_event):
    """
    Stack all the `tel_00X` image tables (in case they exit) and write in the v0.6 file

    Parameters
    input_filename : [ste] input hfile name
    hfile_out : output File pointer
    node_dl1_event : Output hfile (V0.6) dl1.event node pointer
    """
    telescope_node = node_dl1_event.telescope

    imag_per_tels = [Table(table_img.read()) for table_img in telescope_node.images]
    image_table = vstack(imag_per_tels)

    for tab in telescope_node.images:
        hfile_out.remove_node(tab)

    # Todo change names of column `image_mask` to `` ??

    dump_plus_copy_node_to_create_new_table(
        input_filename,
        hfile_out,
        image_table,
        hfile_out.root.dl1.event.telescope.image,
        newname_pointer="LST_LSTCam",
        tmp_name="imgsTable",
    )


def stack_and_write_parameters_table(
    input_filename, hfile_out, node_dl1_event, output_mc_table_pointer
):
    """
    Stack all the `tel_00X` parameters tables (of v0.8), change names of the columns and write the table in the
    V0.6 (lstchain like) format

    Parameters
    hfile_out : output File pointer
    node_dl1_event : Output hfile (V0.6) dl1.event node pointer
    output_mc_table_pointer : output subarray node pointer
    """
    telescope_node = node_dl1_event.telescope

    param_per_tels = [
        Table(table_param.read()) for table_param in telescope_node.parameters
    ]
    parameter_table = vstack(param_per_tels)

    for tab in telescope_node.parameters:
        hfile_out.remove_node(tab)

    parameter_table.rename_column("hillas_intensity", "intensity")
    parameter_table.rename_column("hillas_x", "x")
    parameter_table.rename_column("hillas_y", "y")
    parameter_table.rename_column("hillas_r", "r")
    parameter_table.rename_column("hillas_phi", "phi")
    parameter_table.rename_column("hillas_length", "length")
    parameter_table.rename_column("hillas_width", "width")
    parameter_table.rename_column("hillas_psi", "psi")
    parameter_table.rename_column("hillas_skewness", "skewness")
    parameter_table.rename_column("hillas_kurtosis", "kurtosis")
    parameter_table.rename_column("timing_slope", "time_gradient")
    parameter_table.rename_column("timing_intercept", "intercept")
    parameter_table.rename_column("morphology_num_pixels", "n_pixels")
    parameter_table.rename_column("morphology_num_islands", "n_islands")
    parameter_table.add_column(
        np.log10(parameter_table["intensity"]), name="log_intensity"
    )
    parameter_table.add_column(
        parameter_table["width"] / parameter_table["length"], name="wl"
    )

    # Param table is indeed huge - it contains all the mc_events parameters (from v0.6 !!) too
    mc_event_table = Table(output_mc_table_pointer.mc_shower.read())
    mc_event_table.remove_column("obs_id")
    parameter_table = join(parameter_table, mc_event_table, keys="event_id")
    parameter_table.add_column(
        np.log10(parameter_table["mc_energy"]), name="log_mc_energy"
    )

    dump_plus_copy_node_to_create_new_table(
        input_filename,
        hfile_out,
        parameter_table,
        hfile_out.root.dl1.event.telescope.parameters,
        newname_pointer="LST_LSTCam",
        tmp_name="paramsTable",
    )


def dump_plus_copy_node_to_create_new_table(
    input_filename,
    hfile_out,
    astropy_table_to_copy,
    newparent_pointer,
    newname_pointer,
    tmp_name,
    overwrite=False,
):
    """
    General function to write an astropy table to a temporal file, and immediately after copy it to the
    output v0.6 hfile.

    Parameters
    input_filename : [ste] input hfile name
    hfile_out : output File pointer
    astropy_table_to_copy : astropy table to be copied
    newparent_pointer : newparent copy_node parameter
    newname_pointer : newname copy_node parameter
    tmp_name : [str] flag to identify the temportal table and make it unique (necessary when simultaneous reorganizers
                are run in the same dir)
    overwrite : overwrite parameter of the copy_node method
    """
    input_filename = input_filename.split("___")[0]
    if tmp_name == "":
        flag_name = "UNKNOWN"
    else:
        flag_name = tmp_name

    temp_table_name = f"{input_filename}_tmp_table_reoganizer_{flag_name}.h5"
    write_table_hdf5(astropy_table_to_copy, temp_table_name, path="/root")
    temp_table = tables.open_file(temp_table_name, "r")
    hfile_out.copy_node(
        temp_table.root.root,
        newparent=newparent_pointer,
        newname=newname_pointer,
        overwrite=overwrite,
    )
    temp_table.close()
    os.remove(temp_table_name)


def rename_mc_shower_colnames(
    input_filename, hfile_out, event_node, output_mc_table_pointer
):
    """
    Rename column names of the `mc_shower` table and dump the table to the v0.6 output hfile.

    Parameters
    input_filename : [ste] input hfile name
    hfile_out : output File pointer
    event_node : root.dl1.event node (of output hfile, so V0.6)
    output_mc_table_pointer : output subarray node pointer
    """
    mc_shower_table = Table(event_node.subarray.mc_shower.read())
    mc_shower_table.rename_column("true_energy", "mc_energy")
    mc_shower_table.rename_column("true_alt", "mc_alt")
    mc_shower_table.rename_column("true_az", "mc_az")
    mc_shower_table.rename_column("true_core_x", "mc_core_x")
    mc_shower_table.rename_column("true_core_y", "mc_core_y")
    mc_shower_table.rename_column("true_h_first_int", "mc_h_first_int")
    mc_shower_table.rename_column("true_x_max", "mc_x_max")
    mc_shower_table.rename_column("true_shower_primary_id", "mc_shower_primary_id")

    dump_plus_copy_node_to_create_new_table(
        input_filename,
        hfile_out,
        mc_shower_table,
        output_mc_table_pointer,
        newname_pointer="mc_shower",
        tmp_name="mcShowerTable",
        overwrite=True,
    )


def create_hfile_out(
    input_filename,
    outfile_name,
    sim_pointer08,
    config_pointer08,
    dl1_pointer,
    filter_pointer,
):
    """
    Create output hfile (lstchainv0.6 like hdf5 file)

    Parameters
    input_filename : [ste] input hfile name
    outfile_name : [str] output hfile name
    sim_pointer08 : dl1-file_v0.8_simulation pointer
    config_pointer08 : dl1-file_v.08_configuration pointer
    dl1_pointer :  dl1-file_v0.8_dl1 pointer
    filter_pointer : dl1-file_v0.8 filters pointer
    """
    hfile_out = tables.open_file(outfile_name, "w")
    hfile_out.create_group("/", "simulation")
    hfile_out.create_group("/", "dl1")

    # Simulation node V0.6
    #    /simulation (Group) 'Simulation information of the run'
    #       children := ['mc_event' (Table), 'run_config' (Table), 'thrown_event_distribution' (Table)]
    hfile_out.copy_node(
        sim_pointer08.service.shower_distribution,
        newparent=hfile_out.root.simulation,
        newname="thrown_event_distribution",
        recursive=True,
        filters=filter_pointer,
    )
    hfile_out.copy_node(
        config_pointer08.simulation.run,
        newparent=hfile_out.root.simulation,
        newname="run_config",
        recursive=True,
        filters=filter_pointer,
    )

    # Instrument node V0.6
    #    --instrument (Group)
    #       +--telescope (Group)
    #       |  +--camera (Group)
    #              +--readout_LSTCam --> copied free, it can be erase.
    #              +--geometry_LSTCAM --> To be renamed to LSTCam
    #       |  `--optics (Table)
    #       `--subarray (Group)
    #          `--layout (Table)
    instrument_node = hfile_out.copy_node(
        config_pointer08.instrument,
        newparent=hfile_out.root,
        recursive=True,
        filters=filter_pointer,
    )
    hfile_out.rename_node(
        instrument_node.telescope.camera.geometry_LSTCam, newname="LSTCam"
    )

    # dl1 node V0.6
    #    +--dl1 (Group)
    #       `--event (Group)
    #          +--telescope (Group)
    #             +--image (Group)
    #             `--parameters (Group)
    #          `--subarray (Group)
    #             +--mc_shower (Table)
    #             `--trigger (Table)
    dl1_event_node06 = hfile_out.copy_node(
        dl1_pointer.event,
        newparent=hfile_out.root.dl1,
        recursive=True,
        filters=filter_pointer,
    )
    # This will only happen on ctapipe, not RTA
    # hfile_out.remove_node(dl1_event_node06.telescope.trigger)  # Table stored twice, remove to avoid problems.

    subarray_pointer = hfile_out.root.dl1.event.subarray
    hfile_out.copy_node(
        sim_pointer08.event.subarray.shower,
        newparent=subarray_pointer,
        newname="mc_shower",
        recursive=True,
        filters=filter_pointer,
    )

    rename_mc_shower_colnames(
        input_filename, hfile_out, dl1_event_node06, subarray_pointer
    )
    stack_and_write_parameters_table(
        input_filename, hfile_out, dl1_event_node06, subarray_pointer
    )
    if "image" in dl1_event_node06.telescope:
        stack_and_write_images_table(input_filename, hfile_out, dl1_event_node06)

    hfile_out.close()


def main(input_filename, output_filename):
    """
    Conversion from dl1 data model (ctapipe and hiper(CTA)RTA) data model, and convert it to lstchain_v0.6 data mode.

    Parameters
    input_filename : [str] Input filename
    output_filename : [str] Output filename
    """
    hfile = tables.open_file(input_filename, "r")

    # dl1 v0.8 Pointers
    simulation_v08 = hfile.root.simulation
    configuration_v08 = hfile.root.configuration
    dl1_v08 = hfile.root.dl1
    filter_v08 = hfile.filters

    create_hfile_out(
        input_filename,
        output_filename,
        simulation_v08,
        configuration_v08,
        dl1_v08,
        filter_v08,
    )

    # Add disp_* and mc_type to the parameters table.
    add_disp_and_mc_type_to_parameters_table(
        output_filename, "dl1/event/telescope/parameters/LST_LSTCam"
    )

    hfile.close()


if __name__ == "__main__":
    args = parser.parse_args()
    main(args.infile, args.outfile)



#!/usr/bin/env python
# Licensed under a MIT license - see LICENSE.rst

import os
import json
from pathlib import Path
from setuptools import setup, find_packages


def find_scripts(script_dir, prefix):
    script_list = [f for f in Path(script_dir).iterdir() if f.name.startswith(prefix)]
    return script_list


# Read package info from codemeta.json
with open(os.path.join(os.path.dirname(__file__), "codemeta.json")) as file:
    metadata = json.load(file)

project_name = metadata["name"]
author_names = ""
for aut in metadata["author"]:
    author_names += f"{aut['givenName']} {aut['familyName']},"

version = metadata["version"]
description = metadata["description"]

scripts_list = find_scripts("lstmcpipe", "onsite_")

entry_points = {
    "console_scripts": [
        "lstmcpipe = lstmcpipe.lstmcpipe_start:main",
        "lstmcpipe_plot_models_importance = lstmcpipe.plots.plot_models_importance:main",
        "lstmcpipe_plot_irfs = lstmcpipe.plots.plot_irfs:main",
        "lstmcpipe_hiperta_r0_to_dl1lstchain = lstmcpipe.hiperta.hiperta_r0_to_dl1lstchain:main",
        "lstmcpipe_dl2_to_sensitivity = lstmcpipe.scripts.script_dl2_to_sensitivity:main",
        "lstmcpipe_train_test_split = lstmcpipe.scripts.script_train_test_splitting:main",
        "lstmcpipe_lst_core_r0_dl1 = lstmcpipe.scripts.script_batch_filelist_lst:main",
        "lstmcpipe_lst_core_dl1ab = lstmcpipe.scripts.script_batch_filelist_lst_dl1ab:main",
        "lstmcpipe_cta_core_r0_dl1 = lstmcpipe.scripts.script_batch_filelist_cta:main",
        "lstmcpipe_rta_core_r0_dl1 = lstmcpipe.scripts.script_batch_filelist_rta:main",
        "lstmcpipe_compare_irfs = lstmcpipe.scripts.script_compare_irfs:main",
        "lstmcpipe_validate_config = lstmcpipe.scripts.script_lstmcpipe_validate_config:main",
        "lstmcpipe_generate_config = lstmcpipe.scripts.lstmcpipe_generate_config:main",
    ]
}

setup(
    name=project_name,
    version=version,
    description=description,
    install_requires=[
        "lstchain",
        "numpy",
        "astropy",
        "ctaplot>=0.5",
        "pyirf>=0.4",
        "matplotlib",
        "pytest",
        "ruamel.yaml>=0.17",
        "setuptools_scm",
    ],
    packages=find_packages(exclude="lstmcpipe._dev_version"),
    tests_require=["pytest"],
    author=author_names,
    author_email="thomas.vuillaume@lapp.in2p3.fr",
    url="https://github.com/cta-observatory/lstmcpipe",
    long_description="Visit https://github.com/cta-observatory/lstmcpipe",
    license="MIT",
    classifiers=[
        "Intended Audience :: Science/Research",
        "Programming Language :: Python :: 3",
        "Topic :: Scientific/Engineering :: Astronomy",
    ],
    scripts=scripts_list,
    entry_points=entry_points,
    include_package_data=True,
    package_data={'lstmcpipe': ['codemeta.json']},
    data_files=[('lstmcpipe', ['lstmcpipe/base_config_lstmcpipe.yaml'])],
    use_scm_version={
        "write_to": Path(__file__).parent.joinpath("lstmcpipe/_version.py"),
        "write_to_template": "__version__ = '{version}'",
    },
)



"""
Script to generate a mini production tree with symlinks to a few simtel files and the corresponding path config.
This is used to test lstmcpipe on a small prod
"""

from pathlib import Path
from tqdm import tqdm
from datetime import date
import os
import argparse

from lstmcpipe.config import paths_config



def generate_tree(base_dir, working_dir, nfiles):
    """
    Walk the base dir looking for simtels files
    When a directory contains simtels files, it's tree structure is duplicated into the working dir
    and nfiles are symlinked there
    """
    base_dir = Path(base_dir)
    working_dir = Path(working_dir)
    
    for root, dirs, files in tqdm(os.walk(base_dir)):
        simtel_files = [os.path.join(root, file) for file in files if file.endswith('.simtel.gz')]
        if simtel_files:
            for file in simtel_files[:nfiles]:
                target = working_dir.joinpath(Path(file).relative_to(base_dir))
                target.parent.mkdir(parents=True, exist_ok=True)
                if not target.exists():
                    target.symlink_to(file)



def generate_test_prod5trans80(working_dir='/fefs/aswg/workspace/lstmcpipe/data/mc/',
                               nfiles=5, path_to_config_file='.', overwrite=True):
    base_dir = '/fefs/aswg/workspace/lstmcpipe/data/test_data/mc/DL0/20200629_prod5_trans_80/'
    working_dir = os.path.join(working_dir, 'DL0/20200629_prod5_trans_80/')
    
    generate_tree(base_dir, working_dir, nfiles)

    pc = paths_config.PathConfigProd5Trans80(f'test_prod_{date.today()}')
    pc.base_dir = os.path.join(working_dir, '{data_level}/20200629_prod5_trans_80/{particle}/{zenith}/south_pointing/{prod_id}')
    pc.generate()
    pc.save_yml(os.path.join(path_to_config_file, f'test_prod5trans80_{date.today()}.yaml'), overwrite=overwrite)


def generate_test_allsky(working_dir='/fefs/aswg/workspace/lstmcpipe/data/mc/',
                         nfiles=5, path_to_config_file='.', dec='dec_4822',  overwrite=True):
    allsky_base_dir = '/home/georgios.voutsinas/ws/AllSky/'
    
    generate_tree(allsky_base_dir, os.path.join(working_dir, 'DL0/AllSky'), nfiles)

    pc = paths_config.PathConfigAllSky(f'test_prod_{date.today()}', dec)
    pc.base_dir = os.path.join(working_dir, '{data_level}/AllSky/{prod_id}/{particle}/{pointing}')
    # config training dir are replaced with local ones
    pc.training_dir = os.path.join(working_dir, pc.training_dir.replace(allsky_base_dir, 'DL0/AllSky/'))
    pc.testing_dir = os.path.join(working_dir, pc.testing_dir.replace(allsky_base_dir, 'DL0/AllSky/'))
    pc.generate()
    pc.save_yml(os.path.join(path_to_config_file, f'test_AllSky_{date.today()}.yaml'), overwrite=overwrite)



if __name__ == '__main__':

    parser = argparse.ArgumentParser(description='Generate test tree')

    parser.add_argument('prod_type', type=str, 
                        help='prod5trans80 or allsky')
    parser.add_argument('--nfiles', type=int, default=5,
                        help='Number of files')
    parser.add_argument('--path_config_file', type=Path, default='.',
                        help='Path to save the corresponding config file')
    parser.add_argument('--working_dir', type=Path, default='/fefs/aswg/workspace/lstmcpipe/data/mc/',
                        help='Your working dir where the DL0 tree will be generated, '
                             '(such as /fefs/aswg/workspace/firstname.surname/data/mc/)')

    args = parser.parse_args()
    
    if args.prod_type == 'prod5trans80':
        generate_test_prod5trans80(args.working_dir, args.nfiles, args.path_config_file)
    elif args.prod_type == 'allsky':
        generate_test_allsky(args.working_dir, args.nfiles, args.path_config_file)
    else:
        raise NotImplementedError("Unknown prod type")


# Functions used to manage the MC workflow analysis at La Palma

# Enrique Garcia Nov 2019

import os
from ruamel.yaml import YAML
import shutil
import logging
from pathlib import Path


log = logging.getLogger(__name__)


def save_log_to_file(dictionary, output_file, workflow_step=None):
    """
    Dumps a dictionary (log) into a dicts of dicts with keys each of the pipeline stages.

    Parameters
    ----------
    dictionary : dict
        The dictionary to be dumped to a file
    output_file : str or Path
        Output file to store the log
    workflow_step : str
        Step of the workflow, to be recorded in the log

    Returns
    -------
        None
    """
    if workflow_step is None:
        workflow_step = "NoKEY"

    dict2log = {workflow_step: dictionary}

    with open(output_file, "a+") as fileout:
        YAML().dump(dict2log, fileout)


def batch_mc_production_check(
    dict_jobids_all_stages,
    log_directory,
    prod_id,
    prod_config_file,
    batch_config,
    logs_files,
):
    """
    Check that the dl1_to_dl2 stage, and therefore, the whole workflow has ended correctly.
    The machine information of each job will be dumped to the file.
    The file will take the form `check_MC_prodID_{prod_id}_OK.txt`

    Parameters
    ----------
    dict_jobids_all_stages : dict
        dict containing the {stage: all_job_ids related} information
    log_directory: Path
    prod_id: str
    prod_config_file: str
    batch_config: dict
    logs_files: dict
        Dictionary with logs files

    Returns
    -------
    jobid : str

    """
    debug_log = {}
    all_pipeline_jobs = []

    source_env = batch_config["source_environment"]
    slurm_account = batch_config["slurm_account"]

    for stage, jobids in dict_jobids_all_stages.items():
        all_pipeline_jobs.append(jobids)
        debug_log.update({f"SUMMARY_{stage}": jobids})

    all_pipeline_jobs = ",".join(all_pipeline_jobs)

    # Copy lstmcpipe config used to log directory
    shutil.copyfile(
        Path(prod_config_file).resolve(),
        log_directory.joinpath(f"config_MC_prod_{prod_id}.yml")
    )

    # Save machine info into the check file
    check_prod_file = log_directory.joinpath(f"check_MC_{prod_id}.txt").absolute().as_posix()

    cmd_wrap = f"touch {check_prod_file}; "
    cmd_wrap += (
        f"sacct --format=jobid,jobname,nodelist,cputime,state,exitcode,avediskread,maxdiskread,avediskwrite,"
        f"maxdiskwrite,AveVMSize,MaxVMSize,avecpufreq,reqmem -j {all_pipeline_jobs} >> {check_prod_file}; "
        f"mv slurm-* IRFFITSWriter.provenance.log {log_directory.absolute().as_posix()} "
    )

    batch_cmd = "sbatch -p short --parsable"
    if slurm_account != "":
        batch_cmd += f" -A {slurm_account}"
    batch_cmd += (
        f" --dependency=afterok:{all_pipeline_jobs} -J prod_check"
        f' --wrap="{source_env} {cmd_wrap}"'
    )

    jobid = os.popen(batch_cmd).read().strip("\n")
    log.info(f"Submitted batch CHECK-job {jobid}")
    debug_log.update({f"prod_check_{jobid}": batch_cmd})

    save_log_to_file(debug_log, logs_files["debug_file"],
                     workflow_step="check_full_workflow")

    return jobid



#!/usr/bin/env python

import os
import shutil
import logging
from pathlib import Path
from lstmcpipe.workflow_management import save_log_to_file

log = logging.getLogger(__name__)


def batch_train_test_splitting(
        dict_paths,
        jobids_from_r0dl1,
        batch_config,
        logs,
):
    """

    Parameters
    ----------
    dict_paths: dict
    jobids_from_r0dl1: str
    batch_config: dict
        Dictionary containing the (full) source_environment and the slurm_account strings to be passed
        to `merge_dl1` and `compose_batch_command_of_script` functions.
    logs: dict
        Dictionary with logs files

    Returns
    -------

    """
    log_splitting = {}
    debug_log = {}
    jobids_for_merging = []

    log.info("==== START {} ====".format("batch train_test_splitting"))

    for paths in dict_paths:
        job_logs, jobid = train_test_split(
            paths["input"],
            paths["output"],
            wait_jobid_r0_dl1=jobids_from_r0dl1,
            batch_configuration=batch_config,
            slurm_options=paths.get("slurm_options", None),
        )

        log_splitting.update(job_logs)
        jobids_for_merging.append(jobid)
        debug_log[jobid] = (
            f"Train test splitting jobid that depends on : {jobids_from_r0dl1}"
        )

    jobids_for_merging = ",".join(jobids_for_merging)

    save_log_to_file(log_splitting, logs["log_file"], workflow_step="dl1_to_dl2")
    save_log_to_file(debug_log, logs["debug_file"], workflow_step="dl1_to_dl2")

    log.info("==== END {} ====".format("batch train_test_splitting"))

    return jobids_for_merging


def train_test_split(
        input_dir,
        output_dirs,
        batch_configuration,
        wait_jobid_r0_dl1=None,
        slurm_options=None,
):
    """

    Parameters
    ----------
    input_dir: str
    output_dirs: dict
    batch_configuration: dict
    wait_jobid_r0_dl1: str
    slurm_options: str
        Extra slurm options to be passed to the sbatch command

    Returns
    -------

    """
    source_env = batch_configuration["source_environment"]
    slurm_account = batch_configuration["slurm_account"]

    log_splitting = {}

    input_dir = Path(input_dir).resolve()

    log.info("Splitting files within the {} dir".format(input_dir.as_posix()))

    # create train, test output directories
    test_dir = Path(output_dirs["test"]).resolve()
    train_dir = Path(output_dirs["train"]).resolve()
    for direct in [test_dir, train_dir]:
        if direct.exists() and any(direct.iterdir()):
            shutil.rmtree(direct)
    train_dir.mkdir(exist_ok=True, parents=True)
    test_dir.mkdir(exist_ok=True, parents=True)

    # tt ratio
    if "ratio" not in output_dirs:
        ratio = 0.5
    else:
        ratio = output_dirs["ratio"]

    cmd = f"{source_env} lstmcpipe_train_test_split -i {input_dir} --otest {test_dir}" \
          f" --otrain {train_dir} -r {ratio} -l {test_dir.parent}"

    # TODO check these dirs
    jobe = Path(input_dir).joinpath(input_dir, "split_tt_%j.e")
    jobo = Path(input_dir).joinpath(input_dir, "split_tt_%j.o")

    batch_cmd = "sbatch --parsable"
    if slurm_options is not None:
        batch_cmd += f" {slurm_options}"
    else:
        batch_cmd += " -p short"
    if slurm_account != "":
        batch_cmd += f" -A {slurm_account}"
    if wait_jobid_r0_dl1 is not None:
        batch_cmd += f" --dependency=afterok:{wait_jobid_r0_dl1}"
    batch_cmd += f' -J splitting_train_test -e {jobe} -o {jobo} --wrap="{cmd}"'

    jobid_split = os.popen(batch_cmd).read().strip("\n")
    log_splitting.update({jobid_split: batch_cmd})

    return log_splitting, jobid_split


def check_empty_dir(directory):
    """
    Check if a directory is empty. If not, erase all its content.

    Parameters
    ----------
    directory : Path
        Path to dir to check
    """
    if any(directory.iterdir()):
        for item in directory.iterdir():
            item.unlink()
    else:
        pass



#!/usr/bin/env python

# plot RF features importance

import os
import argparse
import matplotlib.pyplot as plt
from lstchain.visualization.plot_dl2 import plot_models_features_importances


def main():
    parser = argparse.ArgumentParser(
        description="Plot features importance from trained random forests models"
    )
    parser.add_argument("models_path", help="path to the trained models")
    parser.add_argument(
        "--config_file", "-cf", default=None, help="Path to the config file. Optional."
    ),
    args = parser.parse_args()

    plot_models_features_importances(args.models_path, config_file=args.config_file)

    plt.savefig(os.path.join(args.models_path, "features_importance.png"))


if __name__ == "__main__":
    main()



# #!/usr/bin/env python
import argparse
import yaml
from pathlib import Path

# from lstmcpipe.config import load_config
from lstmcpipe.config.pipeline_config import config_valid
from lstchain.io.config import read_configuration_file


def validate_lstmcpipe(filename):
    # config = load_config(filename)
    with open(filename) as f:
        config = yaml.safe_load(f)
    if 'prod_id' not in config:
        raise ValueError(f"No prod ID in config {filename}")
    config_valid(config)


def validate_lstchain(filename):
    read_configuration_file(filename)


def main():
    parser = argparse.ArgumentParser(description="Validate your lstmcpipe config")

    # Required arguments
    parser.add_argument(dest='lstmcpipe_config_filename',
                        type=Path,
                        help='Path to lstmcpipe config file',
                        )

    # optional
    parser.add_argument('--lstchain_config', '-lc',
                        action='store',
                        type=Path,
                        dest='lstchain_config_filename',
                        help='Path to the lstchain config file.',
                        default=None,
                        )

    args = parser.parse_args()

    validate_lstmcpipe(args.lstmcpipe_config_filename)
    if args.lstchain_config_filename is not None:
        validate_lstchain(args.lstchain_config_filename)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

import os
import shutil
import logging
from pathlib import Path
from lstmcpipe.workflow_management import save_log_to_file
from lstmcpipe.io.data_management import check_and_make_dir_without_verification


log = logging.getLogger(__name__)


def batch_train_pipe(
        dict_paths,
        jobids_from_merge,
        config_file,
        batch_config,
        logs
):
    """
    Function to batch the lstchain train_pipe once the proton and gamma-diffuse merge_and_copy_dl1 batched jobs have
    finished.

    Parameters
    ----------
    dict_paths : dict
        Core dictionary with {stage: PATHS} information
    config_file : str
        Path to a configuration file. If none is given, a standard configuration is applied
    jobids_from_merge : str
        string containing the jobids (***ONLY from proton and gamma-diffuse***) from the jobs batched in the
         merge_and_copy_dl1 stage, to be passed to the train_pipe function (as a slurm dependency)
    batch_config : dict
        Dictionary containing the (full) source_environment and the slurm_account strings to be passed to
        the `train_pipe` function.
    logs: dict
        Dictionary with logs files

    Returns
    -------
    jobid_4_dl1_to_dl2 : str
        string containing the jobid to be passed to the next stage of the workflow (as a slurm dependency).
        For the next stage, however, it will be needed TRAIN + MERGED jobs
    """
    log_train = {}
    debug_train = {}
    jobid_for_dl1_to_dl2 = []

    log.info("==== START {} ====".format("batch mc_train_workflow"))

    for paths in dict_paths:

        gamma_dl1_train_file = paths["input"]["gamma"]
        proton_dl1_train_file = paths["input"]["proton"]
        models_dir = paths["output"]

        job_logs, jobid = train_pipe(
            gamma_dl1_train_file,
            proton_dl1_train_file,
            models_dir,
            config_file=config_file,
            batch_configuration=batch_config,
            wait_jobs_dl1=jobids_from_merge,
            slurm_options=paths.get("slurm_options", None),
        )

        log_train.update(job_logs)
        jobid_for_dl1_to_dl2.append(jobid)

        debug_train[jobid] = (
            f"The single jobid from train_pipe that depends of {jobids_from_merge} - merge"
            f"_and_copy jobids"
        )

    jobid_for_dl1_to_dl2 = ",".join(jobid_for_dl1_to_dl2)

    save_log_to_file(log_train, logs["log_file"], workflow_step="train_pipe")
    save_log_to_file(debug_train, logs["debug_file"], workflow_step="train_pipe")

    log.info("==== END {} ====".format("batch mc_train_workflow"))

    return jobid_for_dl1_to_dl2


def batch_plot_rf_features(
        dict_paths,
        config_file,
        batch_configuration,
        train_jobid,
        logs,
    ):
    """
    Batches the plot_model_importance.py script that creates a .png with the RF feature's importance models
    after the RF are trained.
    The plot is saved in the same dir in where the modes are stored.

    Parameters
    ----------
     dict_paths : dict
        Core dictionary with {stage: PATHS} information
    config_file: str
        Path to lstchain config file
    batch_configuration : dict
        Dictionary containing the (full) source_environment and the slurm_account strings.
    train_jobid: str
        Single jobid from training stage.
    logs: dict
        Dictionary with logs files

    Returns
    -------
    jobid : str
        jobid of batched stage
    """
    log_rf_feat = {}
    log_debug = {}
    all_jobs_plot_rf_feat = []

    log.info("==== START {} ====".format("batch plot RF features importance"))

    source_env = batch_configuration["source_environment"]
    slurm_account = batch_configuration["slurm_account"]

    for path in dict_paths:
        models_dir = path["output"]

        jobe = Path(models_dir).joinpath("job_plot_rf_feat_importance.e").resolve().as_posix()
        jobo = Path(models_dir).joinpath(models_dir, "job_plot_rf_feat_importance.o").resolve().as_posix()

        batch_cmd = f"lstmcpipe_plot_models_importance {models_dir} -cf {config_file}"

        slurm_cmd = "sbatch --parsable --mem=16G "
        if slurm_account != "":
            slurm_cmd += f" -A {slurm_account}"
        slurm_cmd += (
            f" --dependency=afterok:{train_jobid} -e {jobe} -o {jobo} "
            f' -J RF_importance --wrap="export MPLBACKEND=Agg; {source_env} {batch_cmd}"'
        )
        jobid = os.popen(slurm_cmd).read().strip("\n")

        log_rf_feat[jobid] = slurm_cmd
        log_debug[jobid] = "Single job_id to plot RF feature s importance"
        all_jobs_plot_rf_feat.append(jobid)

    all_jobs_plot_rf_feat = ','.join(all_jobs_plot_rf_feat)

    save_log_to_file(log_rf_feat, logs["log_file"],
                     workflow_step="plot_RF_features_importance")
    save_log_to_file(log_debug, logs["debug_file"],
                     workflow_step="plot_RF_features_importance")

    log.info(" Random Forest importance's plot will be saved at: {}".format(models_dir))
    log.info("==== END {} ====".format("batch plot RF features importance"))

    return all_jobs_plot_rf_feat


def train_pipe(
    gamma_dl1_train_file,
    proton_dl1_train_file,
    models_dir,
    config_file=None,
    batch_configuration='',
    wait_jobs_dl1=None,
    slurm_options=None,
):
    """
    Train RF from MC DL1 data (onsite LaPalma cluster)

    Parameters
    ----------
    gamma_dl1_train_file: str
        FILE path to the gamma
    proton_dl1_train_file: str
        FILE path to the proton
    models_dir:
        DIR trained models output path
    config_file: str
        Path to a configuration file. If none is given, a standard configuration is applied
    batch_configuration : dict
        Dictionary containing the (full) source_environment and the slurm_account strings to be passed to the
        sbatch commands
        ! NOTE : train_pipe AND dl1_to_dl2 MUST BE RUN WITH THE SAME ENVIRONMENT
    wait_jobs_dl1 : str
        A string (of chained job_ids separated by ',' and without spaces between each element), containing
        all the job_ids of the merging stage
    slurm_options: str
        Extra slurm options to be passed to the sbatch command

    Returns
    -------
    log_train : dict
        dictionary containing the jobid of the batched job as key and the run command (the lstchain_mc_train
        command with all its corresponding arguments) as value.

    jobid_train : str
        jobid of the batched job to be send (for dependencies purposes) to the next stage of the
        workflow (onsite_mc_dl1_to_dl2)
    """
    log_train = {}

    source_environment = batch_configuration["source_environment"]
    slurm_account = batch_configuration["slurm_account"]

    log.info("Models will be placed in {}".format(models_dir))
    check_and_make_dir_without_verification(models_dir)

    cmd = (
        f" {source_environment} lstchain_mc_trainpipe --fg {gamma_dl1_train_file}"
        f" --fp {proton_dl1_train_file} -o {models_dir}"
    )

    if config_file is not None:
        cmd = cmd + " -c {}".format(config_file)

    jobo = Path(models_dir).joinpath("train_job.o").resolve().as_posix()
    jobe = Path(models_dir).joinpath("train_job.e").resolve().as_posix()

    # 'sbatch --parsable --dependency=afterok:{wait_ids_proton_and_gammas} -e {jobe} -o {jobo} --wrap="{base_cmd}"'
    batch_cmd = "sbatch --parsable"
    # For training, we'd need at least 32G (AllSky) and long queue, user can change this value, though.
    if slurm_options is not None:
        batch_cmd += f" {slurm_options}"
    else:
        batch_cmd += " -p long --mem=32G "
    if slurm_account != "":
        batch_cmd += f" -A {slurm_account}"
    if wait_jobs_dl1 != "":
        batch_cmd += " --dependency=afterok:" + wait_jobs_dl1
    batch_cmd += f' -J train_pipe -e {jobe} -o {jobo} --wrap="{cmd}" '

    jobid_train = os.popen(batch_cmd).read().strip("\n")
    log_train[jobid_train] = batch_cmd

    # copy config into working dir
    if config_file is not None:
        shutil.copyfile(
            config_file,
            Path(models_dir).joinpath(Path(config_file).name)
        )

    return log_train, jobid_train
